from Box2D import *
from math import pi

class ebThing():
    def __init__(self, *args, **kwargs):
        pass

    def ApplyLinearImpulse(self, a, b):
        body = self.body
        body.ApplyLinearImpulse(
                impulse=b2Vec2(a,-b),
                point=body.position,
                wake=True,
        )

    def ApplyAngularImpulse(self, imp):
        body = self.body
        self.body.ApplyAngularImpulse(
                impulse=imp,
                wake=True,
        )

    @property
    def worldCenter(self):
        return self.body.worldCenter

    @property
    def position(self):
        return self.body.position

    @property
    def angle(self):
        # angle en radian
        return self.body.transform.angle

    @property
    def angularDamping(self):
        return self.body.angularDamping

    @angularDamping.setter
    def angularDamping(self, damping):
        self.body.angularDamping = damping



from guizero import Drawing
from PIL import Image, ImageTk
from Box2D import *
from .thing import *
from math import pi,sqrt
from time import time

class ebImage(ebThing):
    def __init__(self, box, x, y, width, height, **kwargs):
        world  = box.world
        canvas = box.canvas

        # mémorisation de la box
        self.box   = box

        # paramètres
        self._visible   = kwargs.pop('visible', True)
        self._static    = kwargs.pop('static', False)

        file            = kwargs.pop('image', None)

        angle           = kwargs.pop('angle', 0)
        dens            = kwargs.pop('density', 1)
        fric            = kwargs.pop('friction', 0)
        rest            = kwargs.pop('restitution', 0)

        # body box2d
        if self._static:
            constructor = world.CreateStaticBody
        else:
            constructor = world.CreateDynamicBody
        body = constructor(position = (x/box._scale, y/box._scale))
        body.CreatePolygonFixture(
            box=(width/box._scale/2, height/box._scale/2),
            density=dens,
            friction=fric,
            restitution=rest,
        )
        # item canvas
        if self._visible and file:
            # calcul des dimensions de l'image
            shape  = body.fixtures[0].shape
            coords = shape.vertices
            width,height = coords[2][0]-coords[0][0], coords[2][1]-coords[0][1]
            # lecture de l'image
            img = Image.open(file)
            w,h = img.size
            ratio = (width/w, height/h)
            side  = int(sqrt(w**2+h**2))
            # on étend l'image avec des pixels transparents
            # pour pouvoir la tourner sans perdre de pixels
            new = Image.new('RGBA', (side, side), (0, 0, 0, 0))
            new.paste(img,
                        (int((side-w)/2), int((side-h)/2)),
                        img.convert('RGBA'))
            img = new
            w,h = img.size
            # stockage de l'image
            self._ratio = ratio
            self._img = []
            t0 = time()
            for angle in range(360):
                # rotation
                new = img.rotate(angle)
                # redimensionnement
                new = new.resize(
                        (int(w*ratio[0]*box._scale),int(h*ratio[1]*box._scale)),
                        # Image.NEAREST,
                        # Image.BICUBIC,
                        Image.LANCZOS,
                )
                # stockage
                self._img.append(ImageTk.PhotoImage(new))
            print('{} : {}s'.format(file,
                            str(int(100*(time()-t0))/100))
                )
            # placement de l'image dans le Canvas
            pos  = body.position
            item = canvas.create_image(
                                pos[0]*box._scale,
                                (box._height - pos[1])*box._scale,
                                image=self._img[0]
                    )
        else:
            item   = None

        # on garde les liens
        self.body = body
        body.userData = self
        self.item = item

    def update_item(self):
        if self._visible:
            box    = self.box
            body   = self.body
            canvas = box.canvas
            pos    = body.position
            shape  = body.fixtures[0].shape
            angle  = int(body.transform.angle*180/pi)

            canvas.itemconfigure(self.item, image=self._img[angle])
            canvas.coords(self.item, pos[0]*box._scale, (box._height - pos[1])*box._scale)


from guizero import App, Drawing
from PIL import Image, ImageTk
from Box2D import *
from random import randint

from .rectangle import ebRectangle
from .circle import ebCircle
from .image import ebImage

class enisebox():
    def __init__(self, canvas, *args, **kwargs):
        # paramètres pour le canvas
        canvas.update()
        self._scale     = kwargs.pop('scale', 20)
        self.canvas    = canvas
        self._width     = canvas.winfo_width()/self._scale
        self._height    = canvas.winfo_height()/self._scale
        # contactListener
        self._contact   = kwargs.pop('contact', None)
        if self._contact is not None:
            contactListener = ebContactListener(self._contact)
            kwargs['contactListener'] = contactListener
        # le monde box2d
        self.world     = b2World(**kwargs)

        # les objets du monde
        self.things   = []

    def create_rectangle(self, *args, **kwargs):
        obj = ebRectangle(self, *args, **kwargs)
        self.things.append(obj)
        return obj

    def create_circle(self, *args, **kwargs):
        obj = ebCircle(self, *args, **kwargs)
        self.things.append(obj)
        return obj

    def create_image(self, *args, **kwargs):
        obj = ebImage(self, *args, **kwargs)
        self.things.append(obj)
        return obj

    def itemconfigure(self, thing, **kwargs):
        item = thing.item
        self.canvas.itemconfigure(item, **kwargs)

    def Step(self, *args):
        self.world.Step(*args)
        for obj in self.things:
            obj.update_item()

    def CreateDistanceJoint(self, thingA, thingB, **kwargs):
        self.world.CreateDistanceJoint(
            bodyA=thingA.body,
            bodyB=thingB.body,
            **kwargs,
        )

    def CreateRopeJoint(self, thingA, thingB, **kwargs):
        self.world.CreateRopeJoint(
            bodyA=thingA.body,
            bodyB=thingB.body,
            **kwargs,
        )

    def CreateFrictionJoint(self, thingA, thingB, **kwargs):
        self.world.CreateFrictionJoint(
            bodyA=thingA.body,
            bodyB=thingB.body,
            **kwargs,
        )

    def CreateGearJoint(self, thingA, thingB, **kwargs):
        self.world.CreateGearJoint(
            bodyA=thingA.body,
            bodyB=thingB.body,
            **kwargs,
        )

    def CreateMouseJoint(self, thingA, thingB, **kwargs):
        self.world.CreateMouseJoint(
            bodyA=thingA.body,
            bodyB=thingB.body,
            **kwargs,
        )

    def CreatePrismaticJoint(self, thingA, thingB, **kwargs):
        self.world.CreatePrismaticJoint(
            bodyA=thingA.body,
            bodyB=thingB.body,
            **kwargs,
        )

    def CreatePulleyJoint(self, thingA, thingB, **kwargs):
        self.world.CreatePulleyJoint(
            bodyA=thingA.body,
            bodyB=thingB.body,
            **kwargs,
        )

    def CreateRevoluteJoint(self, thingA, thingB, **kwargs):
        self.world.CreateRevoluteJoint(
            bodyA=thingA.body,
            bodyB=thingB.body,
            **kwargs,
        )

    def CreateWheelJoint(self, thingA, thingB, **kwargs):
        self.world.CreateWheelJoint(
            bodyA=thingA.body,
            bodyB=thingB.body,
            **kwargs,
        )

    def CreateWeldJoint(self, thingA, thingB, **kwargs):
        self.world.CreateWeldJoint(
            bodyA=thingA.body,
            bodyB=thingB.body,
            **kwargs,
        )

    def DestroyThing(self, thing):
        self.canvas.delete(thing.item)
        self.world.DestroyBody(thing.body)
        thing.body = None

class ebContactListener(b2ContactListener):
    def __init__(self, contactManagement):
        b2ContactListener.__init__(self)
        self._contactManagement = contactManagement
    def BeginContact(self, contact):
        thingA = contact.fixtureA.body.userData
        thingB = contact.fixtureB.body.userData
        self._contactManagement(thingA, thingB, 'begin', contact)
    def EndContact(self, contact):
        thingA = contact.fixtureA.body.userData
        thingB = contact.fixtureB.body.userData
        self._contactManagement(thingA, thingB, 'end', contact)
    def PreSolve(self, contact, oldManifold):
        pass
    def PostSolve(self, contact, impulse):
        pass




from guizero import Drawing
from Box2D import *
from .thing import *

class ebCircle(ebThing):
    def __init__(self, box, x, y, r, **kwargs):
        world  = box.world
        canvas = box.canvas

        # mémorisation de la box
        self.box   = box

        # paramètres
        self._visible   = kwargs.pop('visible', True)
        self._static    = kwargs.pop('static', False)

        dens            = kwargs.pop('density', 1)
        fric            = kwargs.pop('friction', 0)
        rest            = kwargs.pop('restitution', 0)

        # body box2d
        if self._static:
            constructor = world.CreateStaticBody
        else:
            constructor = world.CreateDynamicBody
        body = constructor(position=(x/box._scale, box._height - y/box._scale))
        body.CreateCircleFixture(
            radius=r/box._scale,
            density=dens,
            friction=fric,
            restitution=rest,
        )
        # item canvas
        if self._visible:
            pos = body.position
            x = pos[0]*box._scale
            y = (box._height - pos[1])*box._scale
            r = body.fixtures[0].shape.radius*box._scale
            item = canvas.create_oval(x - r, y - r, x + r, y + r, **kwargs)
        else:
            item = None

        # on garde les liens vers ce qui vient d'être créé
        self.body  = body
        body.userData = self
        self.item  = item

    def update_item(self):
        if self._visible:
            box    = self.box
            body   = self.body
            canvas = box.canvas
            pos  = body.position
            x = pos[0]*box._scale
            y = (box._height - pos[1])*box._scale
            r = body.fixtures[0].shape.radius*box._scale
            canvas.coords(self.item, x - r, y - r, x + r, y + r)



from enisebox.enisebox  import enisebox
from enisebox.circle    import ebCircle
from enisebox.image     import ebImage
from enisebox.thing     import ebThing
from enisebox.rectangle import ebRectangle
